/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.PopupMenu;
import android.widget.PopupMenu.OnMenuItemClickListener;

import net.micode.notes.R;

/**
  *
  * @ProjectName:
  * @Package:        net.micode.notes.ui
  * @ClassName:      DropdownMenu
  * @Description:    DropdownMenu是一个下拉菜单，它的作用是提供一种选择多个选项的方式。
  * 当用户点击DropdownMenu时，会弹出一个下拉列表，列出了多个选项供用户选择。
  * 1.笔记分类：用户可以使用DropdownMenu来选择笔记的分类，比如工作、生活、学习等，从而方便地将笔记进行分类管理。
  * 2.笔记排序：用户可以使用DropdownMenu来选择笔记的排序方式，比如按时间、按标题等进行排序，以满足不同排序需求。
  * 3.标签管理：用户可以使用DropdownMenu来选择或添加标签，以对笔记进行更细粒度的分类和标记。
  * @Author:         xumingyang
  * @CreateDate:     2023-12-24 13:57
  * @UpdateUser:     更新者：
  * @UpdateDate:     2023-12-24 13:57
  * @UpdateRemark:   更新说明：
  * @Version:        1.0
 */
public class DropdownMenu {
    private Button mButton;
    private PopupMenu mPopupMenu;
    //声明一个下拉菜单
    private Menu mMenu;

    public DropdownMenu(Context context, Button button, int menuId) {
        mButton = button;
        mButton.setBackgroundResource(R.drawable.dropdown_icon);
        //设置这个view的背景
        mPopupMenu = new PopupMenu(context, mButton);
        mMenu = mPopupMenu.getMenu();
        mPopupMenu.getMenuInflater().inflate(menuId, mMenu);
        //MenuInflater是用来实例化Menu目录下的Menu布局文件
        //根据ID来确认menu的内容选项
        mButton.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                mPopupMenu.show();
            }
        });
    }

    public void setOnDropdownMenuItemClickListener(OnMenuItemClickListener listener) {
        if (mPopupMenu != null) {
            mPopupMenu.setOnMenuItemClickListener(listener);
        }
    }//设置菜单的监听

    public MenuItem findItem(int id) {
        return mMenu.findItem(id);
    }
    //对于菜单选项的初始化，根据索引搜索菜单需要的选项
    public void setTitle(CharSequence title) {
        mButton.setText(title);
    }
}//布局文件，设置标题
