/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.database.Cursor;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CursorAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;

/**     
  * 
  * @ProjectName:    
  * @Package:        net.micode.notes.ui
  * @ClassName:      FoldersListAdapter
  * @Description:    FoldersListAdapter 的作用是作为文件夹列表的适配器，用于管理和显示文件夹列表的数据和视图。
  * @Author:         xumingyang
  * @CreateDate:     2023-12-24 14:05
  * @UpdateUser:     更新者：
  * @UpdateDate:     2023-12-24 14:05
  * @UpdateRemark:   更新说明：
  * @Version:        1.0
 */
public class FoldersListAdapter extends CursorAdapter {
    /*
     * CursorAdapter是Cursor和ListView的接口
	 * FoldersListAdapter继承了CursorAdapter的类
	 * 主要作用是便签数据库和用户的交互
	 * 这里就是用folder（文件夹）的形式展现给用户
     */
    public static final String [] PROJECTION = {
        NoteColumns.ID,
        NoteColumns.SNIPPET
    };//调用数据库中便签的ID和片段

    public static final int ID_COLUMN   = 0;
    public static final int NAME_COLUMN = 1;

     /**
      * @method  FoldersListAdapter
      * @description 用于初始化FoldersListAdapter类的实例。这个类可能是用于管理文件夹列表显示的适配器，其中包含了数据库操作相关的功能
      * @param context 参数通常是指当前的上下文
      * c 参数可能是一个用于查询数据库的Cursor对象。
      */
    public FoldersListAdapter(Context context, Cursor c) {
        super(context, c);
        // TODO Auto-generated constructor stub
    }//数据库操作

     /**
      * @method  newView
      * @description 用于创建新的视图用于显示文件夹列表项。
      * @return 将该视图返回给适配器使用
      */
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {
        return new FolderListItem(context);
    }//创建一个文件夹，对于各文件夹中子标签的初始化

    @Override
    public void bindView(View view, Context context, Cursor cursor) {
        if (view instanceof FolderListItem) {
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName);
        }
    }//将各个布局文件绑定起来

    public String getFolderName(Context context, int position) {
        Cursor cursor = (Cursor) getItem(position);
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }//根据数据库中标签的ID得到标签的各项内容


    private class FolderListItem extends LinearLayout {
        private TextView mName;

        public FolderListItem(Context context) {
            super(context);
            //操作数据库
            inflate(context, R.layout.folder_list_item, this);
            //根据布局文件的名字等信息将其找出来
            mName = (TextView) findViewById(R.id.tv_folder_name);
        }

        public void bind(String name) {
            mName.setText(name);
        }
    }

}
