/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.SearchManager;
import android.appwidget.AppWidgetManager;
import android.content.ContentUris;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Paint;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Editable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.format.DateUtils;
import android.text.style.BackgroundColorSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.model.WorkingNote;
import net.micode.notes.model.WorkingNote.NoteSettingChangedListener;
import net.micode.notes.tool.DataUtils;
import net.micode.notes.tool.ResourceParser;
import net.micode.notes.tool.ResourceParser.TextAppearanceResources;
import net.micode.notes.ui.DateTimePickerDialog.OnDateTimeSetListener;
import net.micode.notes.ui.NoteEditText.OnTextViewChangeListener;
import net.micode.notes.widget.NoteWidgetProvider_2x;
import net.micode.notes.widget.NoteWidgetProvider_4x;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
  *
  * @ProjectName:
  * @Package:        net.micode.notes.ui
  * @ClassName:      NoteEditActivity
  * @Description:     编辑小米便签应用中的便签内容。它向用户提供了一个界面，允许用户输入、编辑和保存文本信息。
 * 具体来说，用户可以在该界面中添加或编辑标题、标签、内容、图片等信息，并将这些信息保存到小米便签数据库中。
 * 还提供了一些常用的编辑功能，如加粗、倾斜、下划线、字体调整、颜色设置、插入图片等，使得用户可以更加方便地对便签内容进行编辑和美化。
  * @Author:         xumingyang
  * @CreateDate:     2024-01-03 8:44
  * @UpdateUser:     更新者：
  * @UpdateDate:     2024-01-03 8:44
  * @UpdateRemark:   更新说明：
  * @Version:        1.0
 */

public class NoteEditActivity extends Activity implements OnClickListener,
        NoteSettingChangedListener, OnTextViewChangeListener {
    //该类主要是针对标签的编辑
    //继承了系统内部许多和监听有关的类
    private class HeadViewHolder {
        public TextView tvModified;
        public EditText editText1;//新增
        public TextView textView;
        public ImageView ivAlertIcon;

        public TextView tvAlertDate;

        public ImageView ibSetBgColor;
    }
    //使用Map实现数据存储
    private static final Map<Integer, Integer> sBgSelectorBtnsMap = new HashMap<Integer, Integer>();
    static {
        sBgSelectorBtnsMap.put(R.id.iv_bg_yellow, ResourceParser.YELLOW);
        sBgSelectorBtnsMap.put(R.id.iv_bg_red, ResourceParser.RED);
        sBgSelectorBtnsMap.put(R.id.iv_bg_blue, ResourceParser.BLUE);
        sBgSelectorBtnsMap.put(R.id.iv_bg_green, ResourceParser.GREEN);
        sBgSelectorBtnsMap.put(R.id.iv_bg_white, ResourceParser.WHITE);
        //put函数是将指定值和指定键相连
    }

    private static final Map<Integer, Integer> sBgSelectorSelectionMap = new HashMap<Integer, Integer>();
    static {
        sBgSelectorSelectionMap.put(ResourceParser.YELLOW, R.id.iv_bg_yellow_select);
        sBgSelectorSelectionMap.put(ResourceParser.RED, R.id.iv_bg_red_select);
        sBgSelectorSelectionMap.put(ResourceParser.BLUE, R.id.iv_bg_blue_select);
        sBgSelectorSelectionMap.put(ResourceParser.GREEN, R.id.iv_bg_green_select);
        sBgSelectorSelectionMap.put(ResourceParser.WHITE, R.id.iv_bg_white_select);
        //put函数是将指定值和指定键相连
    }

    private static final Map<Integer, Integer> sFontSizeBtnsMap = new HashMap<Integer, Integer>();
    static {
        sFontSizeBtnsMap.put(R.id.ll_font_large, ResourceParser.TEXT_LARGE);
        sFontSizeBtnsMap.put(R.id.ll_font_small, ResourceParser.TEXT_SMALL);
        sFontSizeBtnsMap.put(R.id.ll_font_normal, ResourceParser.TEXT_MEDIUM);
        sFontSizeBtnsMap.put(R.id.ll_font_super, ResourceParser.TEXT_SUPER);
        //put函数是将指定值和指定键相连
    }

    private static final Map<Integer, Integer> sFontSelectorSelectionMap = new HashMap<Integer, Integer>();
    static {
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_LARGE, R.id.iv_large_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_SMALL, R.id.iv_small_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_MEDIUM, R.id.iv_medium_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_SUPER, R.id.iv_super_select);
        //put函数是将指定值和指定键相连
    }

    private static final String TAG = "NoteEditActivity";

    private HeadViewHolder mNoteHeaderHolder;

    private View mHeadViewPanel;
    //私有化一个界面操作mHeadViewPanel，对表头的操作
    private View mNoteBgColorSelector;
    //私有化一个界面操作mNoteBgColorSelector，对背景颜色的操作
    private View mFontSizeSelector;
    //私有化一个界面操作mFontSizeSelector，对标签字体的操作
    private EditText mNoteEditor;
    //声明编辑控件，对文本操作
    private View mNoteEditorPanel;
    //私有化一个界面操作mNoteEditorPanel，文本编辑的控制板
    //private WorkingNote mWorkingNote;
    private WorkingNote mWorkingNote;
    //对模板WorkingNote的初始化
    private SharedPreferences mSharedPrefs;
    //私有化SharedPreferences的数据存储方式
    //它的本质是基于XML文件存储key-value键值对数据
    private int mFontSizeId;
    //用于操作字体的大小
    private static final String PREFERENCE_FONT_SIZE = "pref_font_size";

    private static final int SHORTCUT_ICON_TITLE_MAX_LEN = 10;

    public static final String TAG_CHECKED = String.valueOf('\u221A');
    public static final String TAG_UNCHECKED = String.valueOf('\u25A1');

    private LinearLayout mEditTextList;
    //线性布局
    private String mUserQuery;
    private Pattern mPattern;

     /**
      * @method  onCreate
      * @description 描述一下方法的作用方法的主要作用是初始化Activity的状态、资源和界面，并对便签内容进行处理和计数。
      */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setContentView(R.layout.note_edit);
        //调用setContentView方法设置当前Activity显示的布局为note_edit.xml文件。
        if (savedInstanceState == null && !initActivityState(getIntent())) {
            finish();
            return;
        }
        /*
         * 判断savedInstanceState是否为空，并调用initActivityState方法初始化Activity的状态。
         * initActivityState方法会根据传入的Intent信息，判断当前是新建一个便签还是编辑一个已有的便签。
         */
        initResources();//调用initResources方法初始化Activity中的资源，如绑定控件等。
        count();//新增文档计数功能
    }

    /**
     * Current activity may be killed when the memory is low. Once it is killed, for another time
     * user load this activity, we should restore the former state
     */

     /**
      * @method  onRestoreInstanceState
      * @description 该方法的主要作用是在Activity恢复状态时，根据保存的数据重新初始化Activity的状态，
      * 以保证在Activity被销毁后重新创建时能够正确恢复之前的状态。
      */
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (savedInstanceState != null && savedInstanceState.containsKey(Intent.EXTRA_UID)) {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.putExtra(Intent.EXTRA_UID, savedInstanceState.getLong(Intent.EXTRA_UID));
            //将保存在savedInstanceState中的Intent.EXTRA_UID的值作为Long类型的额外数据传递给新的Intent对象。
            if (!initActivityState(intent)) {
                finish();
                return;
                /*
                 * 调用initActivityState方法，将新的Intent传递进去进行初始化Activity的状态。
                 * 如果initActivityState返回false，说明初始化失败，直接销毁Activity并返回。
                 */
            }
            Log.d(TAG, "Restoring from killed activity");
        }//为防止内存不足时程序的终止，在这里有一个保存现场的函数
    }
    private void count(){//用于实时更新字符数的文本视图
        mNoteEditor.addTextChangedListener(new TextWatcher() {
            int currentLength = 0;

            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                mNoteHeaderHolder.textView.setText("字符数：" + currentLength);
            }//在beforeTextChanged()方法中，通过获取当前文本的长度并更新textView的文本来显示当前字符数。

            public void onTextChanged(CharSequence s, int start, int before, int count) {
                currentLength = mNoteHeaderHolder.editText1.getText().length();
            }//在onTextChanged()方法中，通过获取编辑框的文本长度并将其赋值给currentLength变量，以便在afterTextChanged()方法中使用。

            public void afterTextChanged(Editable s) {
                mNoteHeaderHolder.textView.setText("字符数：" + currentLength);
            }//在afterTextChanged()方法中，再次更新textView的文本以显示最新的字符数
        });
    }

     /**
      * @method  initActivityState
      * @description 作用是初始化NoteEditActivity的状态。
      * @param 
      * @return true表示初始化成功 false表示初始化失败
      */
    private boolean initActivityState(Intent intent) {
        /**
         * If the user specified the {@link Intent#ACTION_VIEW} but not provided with id,
         * then jump to the NotesListActivity
         */
        mWorkingNote = null;
        if (TextUtils.equals(Intent.ACTION_VIEW, intent.getAction())) {
            long noteId = intent.getLongExtra(Intent.EXTRA_UID, 0);
            mUserQuery = "";
            //如果用户实例化标签时，系统并未给出标签ID
            /**
             * Starting from the searched result
             */
            //根据键值查找ID
            if (intent.hasExtra(SearchManager.EXTRA_DATA_KEY)) {
                noteId = Long.parseLong(intent.getStringExtra(SearchManager.EXTRA_DATA_KEY));
                mUserQuery = intent.getStringExtra(SearchManager.USER_QUERY);
            }
            //如果ID在数据库中未找到
            if (!DataUtils.visibleInNoteDatabase(getContentResolver(), noteId, Notes.TYPE_NOTE)) {
                Intent jump = new Intent(this, NotesListActivity.class);
                startActivity(jump);
                //程序将跳转到上面声明的intent——jump
                showToast(R.string.error_note_not_exist);
                finish();
                return false;
            }
            //ID在数据库中找到
            else {
                mWorkingNote = WorkingNote.load(this, noteId);
                if (mWorkingNote == null) {
                    Log.e(TAG, "load note failed with note id" + noteId);
                    //打印出红色的错误信息
                    finish();
                    return false;
                }
            }
            //setSoftInputMode——软键盘输入模式
            getWindow().setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN
                            | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE);
        } else if(TextUtils.equals(Intent.ACTION_INSERT_OR_EDIT, intent.getAction())) {
            // New note
            /*
             * intent.getAction()
             * 大多用于broadcast发送广播时给机制（intent）设置一个action，就是一个字符串
             * 用户可以通过receive（接受）intent，通过 getAction得到的字符串，来决定做什么
             */
            long folderId = intent.getLongExtra(Notes.INTENT_EXTRA_FOLDER_ID, 0);
            int widgetId = intent.getIntExtra(Notes.INTENT_EXTRA_WIDGET_ID,
                    AppWidgetManager.INVALID_APPWIDGET_ID);
            int widgetType = intent.getIntExtra(Notes.INTENT_EXTRA_WIDGET_TYPE,
                    Notes.TYPE_WIDGET_INVALIDE);
            int bgResId = intent.getIntExtra(Notes.INTENT_EXTRA_BACKGROUND_ID,
                    ResourceParser.getDefaultBgId(this));
            // intent.getInt（Long、String）Extra是对各变量的语法分析
            // Parse call-record note
            String phoneNumber = intent.getStringExtra(Intent.EXTRA_PHONE_NUMBER);
            long callDate = intent.getLongExtra(Notes.INTENT_EXTRA_CALL_DATE, 0);
            if (callDate != 0 && phoneNumber != null) {
                if (TextUtils.isEmpty(phoneNumber)) {
                    Log.w(TAG, "The call record number is null");
                }
                long noteId = 0;
                if ((noteId = DataUtils.getNoteIdByPhoneNumberAndCallDate(getContentResolver(),
                        phoneNumber, callDate)) > 0) {
                    mWorkingNote = WorkingNote.load(this, noteId);
                    if (mWorkingNote == null) {
                        Log.e(TAG, "load call note failed with note id" + noteId);
                        finish();
                        return false;
                    }
                    //将电话号码与手机的号码簿相关
                } else {
                    mWorkingNote = WorkingNote.createEmptyNote(this, folderId, widgetId,
                            widgetType, bgResId);
                    mWorkingNote.convertToCallNote(phoneNumber, callDate);
                }
            } else {
                mWorkingNote = WorkingNote.createEmptyNote(this, folderId, widgetId, widgetType,
                        bgResId);
            }//创建一个新的WorkingNote

            getWindow().setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE
                            | WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
        } else {
            Log.e(TAG, "Intent not specified action, should not support");
            finish();
            return false;
        }
        mWorkingNote.setOnSettingStatusChangedListener(this);
        return true;
    }

     /**
      * @method  onResume
      * @description 在Activity恢复时重新初始化便签的显示
      */
    @Override
    protected void onResume() {
        super.onResume();
        initNoteScreen();
        //然后调用initNoteScreen方法，该方法会根据mWorkingNote的内容初始化便签的各个UI控件，例如标题栏、编辑区域、图片等。
    }

     /**
      * @method  initNoteScreen
      * @description 根据mWorkingNote的内容重新初始化便签的UI，包括文本样式、编辑区域、背景、修改时间等
      */
    private void initNoteScreen() {
        mNoteEditor.setTextAppearance(this, TextAppearanceResources
                .getTexAppearanceResource(mFontSizeId));
        //首先根据mFontSizeId设置mNoteEditor的文本样式。
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            switchToListMode(mWorkingNote.getContent());
            /*
             * 如果当前便签是“清单”模式，则调用switchToListMode方法将编辑区域切换到“清单”模式，
             * 并将mWorkingNote中的内容填充到编辑区域。
             */
        } else {
            mNoteEditor.setText(getHighlightQueryResult(mWorkingNote.getContent(), mUserQuery));
            mNoteEditor.setSelection(mNoteEditor.getText().length());
        }
        /*
         * 如果当前便签不是“清单”模式，则将mWorkingNote中的内容填充到编辑区域
         * 并调用getHighlightQueryResult方法将搜索关键字高亮显示。
         */
        for (Integer id : sBgSelectorSelectionMap.keySet()) {
            findViewById(sBgSelectorSelectionMap.get(id)).setVisibility(View.GONE);
        }
        mHeadViewPanel.setBackgroundResource(mWorkingNote.getTitleBgResId());
        mNoteEditorPanel.setBackgroundResource(mWorkingNote.getBgColorResId());
        //将所有背景选择器控件隐藏，然后根据mWorkingNote的标题背景和便签背景设置标题栏和编辑区域的背景。
        mNoteHeaderHolder.tvModified.setText(DateUtils.formatDateTime(this,
                mWorkingNote.getModifiedDate(), DateUtils.FORMAT_SHOW_DATE
                        | DateUtils.FORMAT_NUMERIC_DATE | DateUtils.FORMAT_SHOW_TIME
                        | DateUtils.FORMAT_SHOW_YEAR));

        /**
         * TODO: Add the menu for setting alert. Currently disable it because the DateTimePicker
         * is not ready
         */
        showAlertHeader();
    }

     /**
      * @method  showAlertHeader
      * @description 根据mWorkingNote的提醒设置来显示提醒相关的UI，包括提醒日期文本和提醒图标。
      * 如果有设置提醒，则根据当前时间判断是否已过期，并显示相应的文本。如果没有设置提醒，则隐藏相关UI。
      */
    private void showAlertHeader() {
        if (mWorkingNote.hasClockAlert()) {
            //首先判断mWorkingNote是否有设置提醒（hasClockAlert）。如果有设置提醒，则执行以下步骤：
            long time = System.currentTimeMillis();//获取当前时间
            if (time > mWorkingNote.getAlertDate()) {
                mNoteHeaderHolder.tvAlertDate.setText(R.string.note_alert_expired);
                //判断当前时间是否超过了提醒时间（mWorkingNote.getAlertDate）。如果超过了提醒时间，则将提醒日期文本（tvAlertDate）
                // 设置为"已过期"的字符串资源（R.string.note_alert_expired）。
            } else {
                mNoteHeaderHolder.tvAlertDate.setText(DateUtils.getRelativeTimeSpanString(
                        mWorkingNote.getAlertDate(), time, DateUtils.MINUTE_IN_MILLIS));
            }//如果没有超过提醒时间，则将提醒日期文本设置为相对时间，即距离提醒时间还有多久（使用DateUtils.getRelativeTimeSpanString方法）。
            mNoteHeaderHolder.tvAlertDate.setVisibility(View.VISIBLE);
            mNoteHeaderHolder.ivAlertIcon.setVisibility(View.VISIBLE);
            //将提醒日期文本（tvAlertDate）和提醒图标（ivAlertIcon）设置为可见。
        } else {
            mNoteHeaderHolder.tvAlertDate.setVisibility(View.GONE);
            mNoteHeaderHolder.ivAlertIcon.setVisibility(View.GONE);
            //如果mWorkingNote没有设置提醒，则隐藏提醒日期文本和提醒图标。
        };
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        initActivityState(intent);
    }
    //在Activity已经存在于任务栈中时，再次启动该Activity时的回调方法，可以在其中处理相关的逻辑操作。
    @Override
 /**
  * @method  onSaveInstanceState
  * @description 在Activity即将被销毁时的回调方法，用于保存当前Activity的状态，
  * 以便在Activity被恢复时能够正确地加载之前的状态。
  * @return void
  */
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        /**
         * For new note without note id, we should firstly save it to
         * generate a id. If the editing note is not worth saving, there
         * is no id which is equivalent to create new note
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
            //在创建一个新的标签时，先在数据库中匹配
            //如果不存在，那么先在数据库中存储
        }
        outState.putLong(Intent.EXTRA_UID, mWorkingNote.getNoteId());
        Log.d(TAG, "Save working note id: " + mWorkingNote.getNoteId() + " onSaveInstanceState");
    }

    @Override
    //MotionEvent是对屏幕触控的传递机制
    public boolean dispatchTouchEvent(MotionEvent ev) {
        if (mNoteBgColorSelector.getVisibility() == View.VISIBLE
                && !inRangeOfView(mNoteBgColorSelector, ev)) {
            mNoteBgColorSelector.setVisibility(View.GONE);
            return true;
        }//颜色选择器在屏幕上可见

        if (mFontSizeSelector.getVisibility() == View.VISIBLE
                && !inRangeOfView(mFontSizeSelector, ev)) {
            mFontSizeSelector.setVisibility(View.GONE);
            return true;
        }//字体大小选择器在屏幕上可见
        return super.dispatchTouchEvent(ev);
    }

    //对屏幕触控的坐标进行操作
    private boolean inRangeOfView(View view, MotionEvent ev) {
        int []location = new int[2];
        view.getLocationOnScreen(location);
        int x = location[0];
        int y = location[1];
        if (ev.getX() < x
                || ev.getX() > (x + view.getWidth())
                || ev.getY() < y
                || ev.getY() > (y + view.getHeight())) {
                    return false;
                    //如果触控的位置超出了给定的范围，返回false
                }
        return true;
    }

    private void initResources() {
        mHeadViewPanel = findViewById(R.id.note_title);
        mNoteHeaderHolder = new HeadViewHolder();
        mNoteHeaderHolder.tvModified = (TextView) findViewById(R.id.tv_modified_date);
        mNoteHeaderHolder.ivAlertIcon = (ImageView) findViewById(R.id.iv_alert_icon);
        mNoteHeaderHolder.tvAlertDate = (TextView) findViewById(R.id.tv_alert_date);

        mNoteHeaderHolder.editText1 = (EditText) findViewById(R.id.note_edit_view);//新增的
        mNoteHeaderHolder.textView = (TextView) findViewById(R.id.text_num);

        mNoteHeaderHolder.ibSetBgColor = (ImageView) findViewById(R.id.btn_set_bg_color);
        mNoteHeaderHolder.ibSetBgColor.setOnClickListener(this);
        mNoteEditor = (EditText) findViewById(R.id.note_edit_view);
        mNoteEditorPanel = findViewById(R.id.sv_note_edit);
        mNoteBgColorSelector = findViewById(R.id.note_bg_color_selector);
        for (int id : sBgSelectorBtnsMap.keySet()) {
            ImageView iv = (ImageView) findViewById(id);
            iv.setOnClickListener(this);
        }//对标签各项属性内容的初始化

        mFontSizeSelector = findViewById(R.id.font_size_selector);
        for (int id : sFontSizeBtnsMap.keySet()) {
            View view = findViewById(id);
            view.setOnClickListener(this);
        };//对字体大小的选择
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
        mFontSizeId = mSharedPrefs.getInt(PREFERENCE_FONT_SIZE, ResourceParser.BG_DEFAULT_FONT_SIZE);
        /**
         * HACKME: Fix bug of store the resource id in shared preference.
         * The id may larger than the length of resources, in this case,
         * return the {@link ResourceParser#BG_DEFAULT_FONT_SIZE}
         */
        if(mFontSizeId >= TextAppearanceResources.getResourcesSize()) {
            mFontSizeId = ResourceParser.BG_DEFAULT_FONT_SIZE;
        }
        mEditTextList = (LinearLayout) findViewById(R.id.note_edit_list);
    }

    @Override
    //在Activity即将失去焦点并进入后台时的回调方法，用于执行一些与保存数据、清除状态等相关的操作。
    protected void onPause() {
        super.onPause();
        if(saveNote()) {
            Log.d(TAG, "Note data was saved with length:" + mWorkingNote.getContent().length());
        }
        clearSettingState();
        //调用clearSettingState()方法来清除一些设置状态。具体的逻辑需要根据实际需求来实现，可能包括清除临时数据、重置一些标志位等。
    }

    //和桌面小工具的同步
    private void updateWidget() {
        Intent intent = new Intent(AppWidgetManager.ACTION_APPWIDGET_UPDATE);
        if (mWorkingNote.getWidgetType() == Notes.TYPE_WIDGET_2X) {
            intent.setClass(this, NoteWidgetProvider_2x.class);
        } else if (mWorkingNote.getWidgetType() == Notes.TYPE_WIDGET_4X) {
            intent.setClass(this, NoteWidgetProvider_4x.class);
        } else {
            Log.e(TAG, "Unspported widget type");
            return;
        }

        intent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS, new int[] {
            mWorkingNote.getWidgetId()
        });
       //将当前Note笔记的Widget ID放入Intent的Extra参数中。注意，这里只更新当前Note笔记所绑定的Widget，因此只需要传入一个Widget ID。
        sendBroadcast(intent);
        setResult(RESULT_OK, intent);
        //通过调用sendBroadcast(intent)方法发送广播，通知系统更新Widget。同时，通过setResult(RESULT_OK, intent)方法将更新完成的结果返回给调用方。
    }//通过创建Intent对象、设置Widget Provider、放入Widget ID等操作，最终将更新完成的结果返回给调用方。

    public void onClick(View v) {
        int id = v.getId();
        //通过v.getId()获取点击事件的View的ID。然后根据不同的ID进行不同的逻辑处理。
        if (id == R.id.btn_set_bg_color) {
            mNoteBgColorSelector.setVisibility(View.VISIBLE);
            findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(View.VISIBLE);
        /*
         * 如果点击的是"btn_set_bg_color"按钮（即设置背景颜色的按钮），则将背景颜色选择器（mNoteBgColorSelector）设置为可见，
         * 并将当前Note笔记的背景颜色对应的选择项设置为可见。
         */
        } else if (sBgSelectorBtnsMap.containsKey(id)) {
            findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(
                    View.GONE);
            mWorkingNote.setBgColorId(sBgSelectorBtnsMap.get(id));
            mNoteBgColorSelector.setVisibility(View.GONE);
        /*
         * 如果点击的ID在sBgSelectorBtnsMap中存在（即点击的是背景颜色选择项的按钮），则将当前Note笔记的背景颜色ID更新为对应的值，
         * 并隐藏之前选中的背景颜色选择项，同时将背景颜色选择器隐藏。
         */
        } else if (sFontSizeBtnsMap.containsKey(id)) {
            findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.GONE);
            mFontSizeId = sFontSizeBtnsMap.get(id);
            mSharedPrefs.edit().putInt(PREFERENCE_FONT_SIZE, mFontSizeId).commit();
            findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.VISIBLE);
            if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
                getWorkingText();
                switchToListMode(mWorkingNote.getContent());
            } else {
                mNoteEditor.setTextAppearance(this,
                        TextAppearanceResources.getTexAppearanceResource(mFontSizeId));
            }
            /*
             * 如果点击的ID在sFontSizeBtnsMap中存在（即点击的是字体大小选择项的按钮），则将当前字体大小的选择项设置为不可见，
             * 将字体大小ID更新为对应的值，并将新的字体大小选择项设置为可见。
             * 同时，根据Note笔记的模式（是否为检查列表模式）进行相应的处理：
             * 如果是检查列表模式，则切换到列表模式并获取最新的Note内容；
             * 否则，仅更新文本编辑器的字体样式。
             */
            mFontSizeSelector.setVisibility(View.GONE);
        }
    }

    @Override
    public void onBackPressed() {
        if(clearSettingState()) {
            return;
        }

        saveNote();
        super.onBackPressed();
    }
    //这段代码重写了Activity的返回按钮操作，
    // 在返回操作中处理了设置状态的清除和Note笔记的保存。


     /**
      * @method  clearSettingState
      * @description 这个方法的作用是在用户按下返回按钮时，检查当前是否处于设置状态（例如正在选择背景颜色或字体大小），
      * 如果是，则清除设置状态并返回true，否则返回false。
      * @return true表示成功清除了设置状态 返回false表示当前没有处于设置状态。
      */
    private boolean clearSettingState() {
        if (mNoteBgColorSelector.getVisibility() == View.VISIBLE) {
            mNoteBgColorSelector.setVisibility(View.GONE);
            return true;
            //首先检查背景颜色选择器（mNoteBgColorSelector）和字体大小选择器（mFontSizeSelector）的可见性
        } else if (mFontSizeSelector.getVisibility() == View.VISIBLE) {
            mFontSizeSelector.setVisibility(View.GONE);
            return true;
        }
        return false;
    }

     /**
      * @method  onBackgroundColorChanged
      * @description 在背景颜色发生变化时，根据新的背景颜色ID更新选中状态的选择项的可见性，
      * 并将新的背景颜色资源应用到相关的视图上，以实现界面的背景颜色变更
      */
    public void onBackgroundColorChanged() {
        findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(
                View.VISIBLE);
        mNoteEditorPanel.setBackgroundResource(mWorkingNote.getBgColorResId());
        mHeadViewPanel.setBackgroundResource(mWorkingNote.getTitleBgResId());
        //获取当前Note笔记的背景颜色资源ID，并将该资源作为背景设置给mNoteEditorPanel和mHeadViewPanel，实现对编辑区域和标题区域的背景颜色的更新。
    }

    public void onTopChanged(String top){
        mWorkingNote.getTopId();
    }
     /**
      * @method  onPrepareOptionsMenu
      * @description 根据当前Note笔记的状态和属性，动态加载选项菜单的内容，并更新其中的部分选项的名称和可见性。
      */
    @Override
    //对选择菜单的准备
    public boolean onPrepareOptionsMenu(Menu menu) {
        if (isFinishing()) {
            return true;
        }//判断当前Activity是否正在关闭，如果是，则直接返回true，不做任何处理。
        clearSettingState();
        menu.clear();
        //调用clearSettingState()方法清除当前的设置状态，并使用menu.clear()方法清空选项菜单中的所有选项。
        if (mWorkingNote.getFolderId() == Notes.ID_CALL_RECORD_FOLDER) {
            getMenuInflater().inflate(R.menu.call_note_edit, menu);
            // MenuInflater是用来实例化Menu目录下的Menu布局文件的
        } else {
            getMenuInflater().inflate(R.menu.note_edit, menu);
        }
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            menu.findItem(R.id.menu_list_mode).setTitle(R.string.menu_normal_mode);
        } else {
            menu.findItem(R.id.menu_list_mode).setTitle(R.string.menu_list_mode);
        }
        if(mWorkingNote.getTopId() == 1){
            menu.findItem(R.id.menu_set_top).setVisible(false);
        } else if (mWorkingNote.getTopId()==0) {
            menu.findItem(R.id.menu_cancel_top).setVisible(false);
        }
        if (mWorkingNote.hasClockAlert()) {
            menu.findItem(R.id.menu_alert).setVisible(false);
        } else {
            menu.findItem(R.id.menu_delete_remind).setVisible(false);
        }//如果当前Note笔记已经有闹钟提醒，则隐藏“添加提醒”选项，否则隐藏“删除提醒”选项。
        return true;
    }

    @Override
     /**
      * @method  onOptionsItemSelected
      * @description 用于处理选项菜单中的各个选项被点击时的事件响应。
      * @return boolean 返回true表示已经处理了该选项的点击事件。
      */
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        //在这个方法中，首先获取被点击的选项的ID，然后根据ID的不同执行相应的操作。
        if (itemId == R.id.menu_new_note) {
            createNewNote();
        //如果被点击的选项是“新建笔记”，则调用createNewNote()方法创建一个新的笔记。
        } else if (itemId == R.id.menu_delete) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle(getString(R.string.alert_title_delete));
            builder.setIcon(android.R.drawable.ic_dialog_alert);
            builder.setMessage(getString(R.string.alert_message_delete_note));
            builder.setPositiveButton(android.R.string.ok,
                    new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            deleteCurrentNote();
                            finish();
                        }
                    });
            builder.setNegativeButton(android.R.string.cancel, null);
            builder.show();
        }
        /*
         * 如果被点击的选项是“删除”，则弹出一个警告对话框，询问用户是否确认删除当前笔记。
         * 如果用户确认删除，则调用deleteCurrentNote()方法删除当前笔记，并关闭当前Activity。
         */
        else if (itemId == R.id.menu_font_size) {//修改字体的大小
            mFontSizeSelector.setVisibility(View.VISIBLE);
            findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.VISIBLE);
        }//如果被点击的选项是“修改字体大小”，则显示字体大小选择器，以供用户选择字体大小。
        else if (itemId == R.id.menu_set_top){
            mWorkingNote.setTop((mWorkingNote.getTopId())==1 ? "0" : "1");
        }
        /*else if (itemId == R.id.menu_cancel_top){
            mWorkingNote.setTop((mWorkingNote.getTopId() ==0 ? "1" : "0"));
        }*/
        else if (itemId == R.id.menu_list_mode) {
            mWorkingNote.setCheckListMode(mWorkingNote.getCheckListMode() == 0 ?
                    TextNote.MODE_CHECK_LIST : 0);
        }//如果被点击的选项是“切换列表模式”，则切换当前笔记的检查列表模式状态。
        else if (itemId == R.id.menu_share) {
            getWorkingText();
            sendTo(this, mWorkingNote.getContent());
        }//如果被点击的选项是“分享”，则获取当前笔记的内容并将其发送到其他应用程序。
        else if (itemId == R.id.menu_send_to_desktop) {
            sendToDesktop();
        }//如果被点击的选项是“发送到桌面”，则将当前笔记发送到桌面。
        else if (itemId == R.id.menu_alert) {
            setReminder();
        }//如果被点击的选项是“添加提醒”，则设置当前笔记的提醒时间
        else if (itemId == R.id.menu_delete_remind) {
            mWorkingNote.setAlertDate(0, false);
        }//如果被点击的选项是“删除提醒”，则清除当前笔记的提醒时间
        return true;
    }

    private void setReminder() {
        DateTimePickerDialog d = new DateTimePickerDialog(this, System.currentTimeMillis());
        d.setOnDateTimeSetListener(new OnDateTimeSetListener() {
            //首先创建一个DateTimePickerDialog对象，并将当前系统时间作为初始时间传入。
            // 然后，为该对话框设置一个日期时间选择器监听器。
            public void OnDateTimeSet(AlertDialog dialog, long date) {
                mWorkingNote.setAlertDate(date	, true);
            }
        });
        d.show();
        //最后，显示日期时间选择器对话框。
    }

    /**
     * Share note to apps that support {@link Intent#ACTION_SEND} action
     * and {@text/plain} type
     */


     /**
      * @method  sendTo
      * @description 用于在给定的Context上下文环境中，利用系统默认的分享功能将特定的文本信息发送给其他应用程序。
      */
    private void sendTo(Context context, String info) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        //在方法内部，首先创建一个新的Intent对象，并将其动作设置为Intent.ACTION_SEND，表示发送内容。
        intent.putExtra(Intent.EXTRA_TEXT, info);
        //接着，通过putExtra方法将要分享的文本信息info放入Intent中，使用Intent.EXTRA_TEXT作为键。
        intent.setType("text/plain");
        //然后，使用setType方法将要分享的内容的MIME类型设置为"text/plain"，表示纯文本类型。
        context.startActivity(intent);
        //最后，通过context.startActivity(intent)启动该Intent，将文本信息发送给其他应用程序进行处理。
    }

    private void createNewNote() {//新建便签
        // Firstly, save current editing notes
        saveNote();

        // For safety, start a new NoteEditActivity
        finish();
        Intent intent = new Intent(this, NoteEditActivity.class);
        intent.setAction(Intent.ACTION_INSERT_OR_EDIT);
        intent.putExtra(Notes.INTENT_EXTRA_FOLDER_ID, mWorkingNote.getFolderId());
        //然后创建一个新的Intent对象，指定要启动的NoteEditActivity类，并设置Action为Intent.ACTION_INSERT_OR_EDIT，
        // 并通过Extra传递当前便签所在文件夹的ID。
        startActivity(intent);
        //启动新的NoteEditActivity。
    }

    private void deleteCurrentNote() {//删除便签
        if (mWorkingNote.existInDatabase()) {
            HashSet<Long> ids = new HashSet<Long>();
            //首先判断当前便签是否存在于数据库中。如果存在，则获取当前便签的ID，并将其添加到一个HashSet对象中。
            long id = mWorkingNote.getNoteId();
            if (id != Notes.ID_ROOT_FOLDER) {
                ids.add(id);
            } else {
                Log.d(TAG, "Wrong note id, should not happen");
            }
            if (!isSyncMode()) {
                if (!DataUtils.batchDeleteNotes(getContentResolver(), ids)) {
                    Log.e(TAG, "Delete Note error");
                }//果不是同步模式，则调用DataUtils.batchDeleteNotes()方法批量删除便签；
            } else {
                if (!DataUtils.batchMoveToFolder(getContentResolver(), ids, Notes.ID_TRASH_FOLER)) {
                    Log.e(TAG, "Move notes to trash folder error, should not happens");
                }//如果是同步模式，则调用DataUtils.batchMoveToFolder()方法将便签移动到回收站文件夹中。
            }
        }
        mWorkingNote.markDeleted(true);
        //调用mWorkingNote.markDeleted(true)方法将当前便签标记为已删除状态。
    }

     /**
      * @method  isSyncMode
      * @description 用于判断是否处于同步模式。
      * @return 如果此名称的长度大于0，则说明处于同步模式，返回true；否则返回false。
      */
    private boolean isSyncMode() {
        return NotesPreferenceActivity.getSyncAccountName(this).trim().length() > 0;
    }


    public void onClockAlertChanged(long date, boolean set) {
        /**
         * User could set clock to an unsaved note, so before setting the
         * alert clock, we should save the note first
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
            //首先检查当前工作的便签是否存在于数据库中，如果不存在，则调用saveNote()方法保存便签。
        }
        if (mWorkingNote.getNoteId() > 0) {
            Intent intent = new Intent(this, AlarmReceiver.class);
            intent.setData(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mWorkingNote.getNoteId()));
            PendingIntent pendingIntent = PendingIntent.getBroadcast(this, 0, intent, 0);
            AlarmManager alarmManager = ((AlarmManager) getSystemService(ALARM_SERVICE));
            showAlertHeader();
            if(!set) {
                alarmManager.cancel(pendingIntent);
            } else {
                alarmManager.set(AlarmManager.RTC_WAKEUP, date, pendingIntent);
            }//果当前便签的ID大于0，则创建一个用于触发闹钟的Intent，并设置相应的PendingIntent。
            // 然后根据参数set来决定是取消闹钟还是设置新的闹钟时间。
        } else {
            /**
             * There is the condition that user has input nothing (the note is
             * not worthy saving), we have no note id, remind the user that he
             * should input something
             */
            Log.e(TAG, "Clock alert setting error");
            showToast(R.string.error_note_empty_for_clock);
            //如果当前便签的ID不大于0，即便签内容为空，会记录错误日志并提示用户应该输入内容。
        }
    }

    public void onWidgetChanged() {
        updateWidget();
    }//更新小部件的显示内容。具体的更新逻辑可能包括重新加载数据、刷新UI等操作，以确保小部件展示的内容和状态是最新的。

     /**
      * @method  onEditTextDelete
      * @description 用于处理删除文本编辑框的操作。
      */
    public void onEditTextDelete(int index, String text) {
        int childCount = mEditTextList.getChildCount();
        if (childCount == 1) {
            return;
        }//首先获取文本编辑框的数量，如果仅剩一个编辑框，则直接返回，不执行删除操作。

        for (int i = index + 1; i < childCount; i++) {
            ((NoteEditText) mEditTextList.getChildAt(i).findViewById(R.id.et_edit_text))
                    .setIndex(i - 1);
        }//从指定索引位置往后遍历所有的文本编辑框，将它们的索引减1，以保持正确的索引关联。

        mEditTextList.removeViewAt(index);
        NoteEditText edit = null;
        if(index == 0) {
            edit = (NoteEditText) mEditTextList.getChildAt(0).findViewById(
                    R.id.et_edit_text);
        } else {
            edit = (NoteEditText) mEditTextList.getChildAt(index - 1).findViewById(
                    R.id.et_edit_text);
        }//通过索引位置从mEditTextList中移除对应的文本编辑框。
        //根据不同的索引位置找到相应的文本编辑框，并将传入的text添加到编辑框的末尾。
        int length = edit.length();
        edit.append(text);
        edit.requestFocus();
        edit.setSelection(length);
        //请求编辑框获取焦点，并将光标定位到最后添加的文本之后。
    }

     /**
      * @method  onEditTextEnter
      * @description 用于处理插入文本编辑框的操作。
      */
    public void onEditTextEnter(int index, String text) {
        /**
         * Should not happen, check for debug
         */
        if(index > mEditTextList.getChildCount()) {
            Log.e(TAG, "Index out of mEditTextList boundrary, should not happen");
        }
        //在方法中，首先检查传入的索引位置是否超出了mEditTextList的子视图数量。如果超出了范围，会记录错误日志。
        View view = getListItem(text, index);
        //根据传入的文本和索引位置创建一个新的视图view。
        mEditTextList.addView(view, index);
        //将创建的视图插入到mEditTextList中的指定索引位置。
        NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
        edit.requestFocus();
        edit.setSelection(0);
        //获取新插入的文本编辑框，并将其请求获取焦点。同时，将光标定位在文本编辑框的开头位置。
        for (int i = index + 1; i < mEditTextList.getChildCount(); i++) {
            ((NoteEditText) mEditTextList.getChildAt(i).findViewById(R.id.et_edit_text))
                    .setIndex(i);
        }
    }

    //用于切换到列表模式
    private void switchToListMode(String text) {
        mEditTextList.removeAllViews();
        //移除所有的视图mEditTextList，以确保列表中没有旧的文本编辑框
        String[] items = text.split("\n");
        int index = 0;
        for (String item : items) {
            if(!TextUtils.isEmpty(item)) {
                mEditTextList.addView(getListItem(item, index));
                index++;
            }
        }//通过将文本使用换行符\n进行分割，得到每行的文本项数组items。
        mEditTextList.addView(getListItem("", index));
        mEditTextList.getChildAt(index).findViewById(R.id.et_edit_text).requestFocus();
        //遍历每个文本项，并检查是否为空。如果不为空，则创建对应的视图getListItem(item, index)并添加到mEditTextList中，
        // 然后递增索引值index。
        mNoteEditor.setVisibility(View.GONE);
        mEditTextList.setVisibility(View.VISIBLE);
        //将mNoteEditor设置为不可见，将mEditTextList设置为可见，从而实现从编辑器模式到列表模式的切换。
    }

    //用于将匹配到的查询结果进行高亮处理。
    private Spannable getHighlightQueryResult(String fullText, String userQuery) {
        SpannableString spannable = new SpannableString(fullText == null ? "" : fullText);
        //创建一个SpannableString对象spannable，将传入的完整文本fullText赋值给它（如果fullText为空，则赋值为空字符串）。
        if (!TextUtils.isEmpty(userQuery)) {
            mPattern = Pattern.compile(userQuery);
            //判断用户的查询词userQuery是否为空。如果不为空，则使用Pattern.compile(userQuery)创建一个正则表达式模式mPattern。
            Matcher m = mPattern.matcher(fullText);
            int start = 0;
            //通过mPattern.matcher(fullText)获取一个Matcher对象m，并设置起始位置start为0
            while (m.find(start)) {
                spannable.setSpan(
                        new BackgroundColorSpan(this.getResources().getColor(
                                R.color.user_query_highlight)), m.start(), m.end(),
                        Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
                //使用while (m.find(start))循环查找匹配的结果。在每次找到匹配时，
                // 使用spannable.setSpan()方法将匹配结果的背景色设置为用户查询高亮的颜色。
                start = m.end();
                //更新start为匹配结果的结束位置，继续查找下一个匹配。
            }
        }
        return spannable;
    }

     /**
      * @method  getListItem
      * @description 用于根据给定的文本项item和索引index创建一个列表项的视图。
      */
    private View getListItem(String item, int index) {
        View view = LayoutInflater.from(this).inflate(R.layout.note_edit_list_item, null);
        //首先使用LayoutInflater.from(this).inflate(R.layout.note_edit_list_item, null)从布局文件note_edit_list_item中获取一个视图view。
        final NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
        edit.setTextAppearance(this, TextAppearanceResources.getTexAppearanceResource(mFontSizeId));
        //从view中获取一个NoteEditText编辑框对象edit，并设置其文本外观为当前字体大小mFontSizeId对应的样式。
        CheckBox cb = ((CheckBox) view.findViewById(R.id.cb_edit_item));
        //从view中获取一个CheckBox对象cb，并设置其选中状态改变的监听器，以便在状态改变时修改编辑框的文本外观。
        cb.setOnCheckedChangeListener(new OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    edit.setPaintFlags(edit.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
                } else {
                    edit.setPaintFlags(Paint.ANTI_ALIAS_FLAG | Paint.DEV_KERN_TEXT_FLAG);
                }
            }
        });

        if (item.startsWith(TAG_CHECKED)) {
            cb.setChecked(true);
            edit.setPaintFlags(edit.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
            item = item.substring(TAG_CHECKED.length(), item.length()).trim();
        } else if (item.startsWith(TAG_UNCHECKED)) {
            cb.setChecked(false);
            edit.setPaintFlags(Paint.ANTI_ALIAS_FLAG | Paint.DEV_KERN_TEXT_FLAG);
            item = item.substring(TAG_UNCHECKED.length(), item.length()).trim();
        }
        /*
        根据文本项的前缀标记判断当前列表项是否被选中。如果是，则将复选框设置为选中状态，并将编辑框的文本外观设置为带有删除线的样式；
        否则，将复选框设置为未选中状态，并将编辑框的文本外观设置为普通样式。
        同时，去除前缀标记并去除两端的空格。
         */

        edit.setOnTextViewChangeListener(this);
        edit.setIndex(index);
        edit.setText(getHighlightQueryResult(item, mUserQuery));
        //设置编辑框的值为经过高亮处理的文本项item，并设置编辑框的索引值为传入的索引index，
        return view;
        // 最后将edit对象添加到视图view中并返回该视图。
    }

    /**
     * @method  onCheckListModeChanged
     * @description 用于监听编辑框中文本的改变，并根据是否有文本来控制复选框的可见性。
     */
    public void onTextChange(int index, boolean hasText) {
        if (index >= mEditTextList.getChildCount()) {
            Log.e(TAG, "Wrong index, should not happen");
            return;
            //判断传入的索引值index是否超出了编辑框列表中子视图的数量范围。
            //如果超出，则打印错误日志并退出方法。
        }
        if(hasText) {
            mEditTextList.getChildAt(index).findViewById(R.id.cb_edit_item).setVisibility(View.VISIBLE);
        } else {
            mEditTextList.getChildAt(index).findViewById(R.id.cb_edit_item).setVisibility(View.GONE);
        }
        //根据传入的hasText参数判断当前编辑框是否有文本。如果有文本，则将该编辑框所在的列表项的复选框设置为可见状态；
        // 否则，将复选框设置为不可见状态。
    }

 /**
  * @method  onCheckListModeChanged
  * @description 用于监听检查列表模式的改变，并根据新的模式来更新界面显示。
  */
    public void onCheckListModeChanged(int oldMode, int newMode) {
        if (newMode == TextNote.MODE_CHECK_LIST) {
            switchToListMode(mNoteEditor.getText().toString());
        }
        //判断新的模式是否为检查列表模式。如果是，则调用switchToListMode()方法，
        // 并传入当前编辑器中的文本内容，以切换到列表模式。
        else {
            if (!getWorkingText()) {
                mWorkingNote.setWorkingText(mWorkingNote.getContent().replace(TAG_UNCHECKED + " ",
                        ""));
            }//调用getWorkingText()方法来获取当前的工作文本。如果获取失败（即没有工作文本），则将工作文本设置为去除了未选中标记的内容。
            mNoteEditor.setText(getHighlightQueryResult(mWorkingNote.getContent(), mUserQuery));
            //将编辑器的文本设置为经过高亮处理的工作文本内容
            mEditTextList.setVisibility(View.GONE);
            //将编辑框列表设置为不可见状态。
            mNoteEditor.setVisibility(View.VISIBLE);
            //将编辑器设置为可见状态。
        }
    }

     /**
      * @method  getWorkingText
      * @description 用于获取工作笔记的文本内容，并根据是否为待办事项模式进行相应处理。
      * @return 该方法返回一个布尔值，表示在待办事项模式下是否有选中的项目。
      */
    private boolean getWorkingText() {
        boolean hasChecked = false;
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < mEditTextList.getChildCount(); i++) {
                View view = mEditTextList.getChildAt(i);
                NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
                if (!TextUtils.isEmpty(edit.getText())) {
                    if (((CheckBox) view.findViewById(R.id.cb_edit_item)).isChecked()) {
                        sb.append(TAG_CHECKED).append(" ").append(edit.getText()).append("\n");
                        hasChecked = true;
                    } else {
                        sb.append(TAG_UNCHECKED).append(" ").append(edit.getText()).append("\n");
                    }
                }
            }
            //检查当前工作笔记mWorkingNote是否为待办事项模式。如果是，就遍历所有子视图view，其中包含每个待办事项的复选框和文本编辑框。
            // 对于每个非空的文本编辑框，如果其相应的复选框被选中，则将文本内容作为已选中的待办事项添加到字符串缓冲区sb中，并将标记hasChecked设置为true；
            // 否则，将文本内容作为未选中的待办事项添加到字符串缓冲区sb中。
            mWorkingNote.setWorkingText(sb.toString());
            //将字符串缓冲区sb中的内容作为工作笔记的文本内容设置到mWorkingNote对象中。
        } else {
            mWorkingNote.setWorkingText(mNoteEditor.getText().toString());
            //如果不是待办事项模式，则将文本编辑器mNoteEditor中的文本内容设置为工作笔记的文本内容。
        }
        return hasChecked;//该方法返回一个布尔值hasChecked，表示在待办事项模式下是否有选中的项目。
    }


    //用于保存当前正在编辑的笔记。
    private boolean saveNote() {
        getWorkingText();
        boolean saved = mWorkingNote.saveNote();
        //在方法内部，首先调用getWorkingText方法获取当前正在编辑的笔记的文本内容，并将其保存到mWorkingNote对象中。
        if (saved) {
            /**
             * There are two modes from List view to edit view, open one note,
             * create/edit a node. Opening node requires to the original
             * position in the list when back from edit view, while creating a
             * new node requires to the top of the list. This code
             * {@link #RESULT_OK} is used to identify the create/edit state
             */
            setResult(RESULT_OK);
            //如果保存成功，则调用setResult(RESULT_OK)方法，设置当前Activity的返回状态为RESULT_OK。
            //这个状态用于标识从编辑状态返回到列表视图时，是创建新笔记还是编辑已有笔记。
        }
        return saved;//调用mWorkingNote.saveNote()方法将笔记保存到数据库中，并将保存结果保存到saved变量中。

    }

    //用于将笔记发送到桌面。
    private void sendToDesktop() {
        /**
         * Before send message to home, we should make sure that current
         * editing note is exists in databases. So, for new note, firstly
         * save it
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
        }//检查当前正在编辑的笔记是否存在于数据库中。如果笔记不存在于数据库中，则调用saveNote()方法保存笔记。

        if (mWorkingNote.getNoteId() > 0) {
            Intent sender = new Intent();
            //创建一个Intent对象sender用于发送广播。
            Intent shortcutIntent = new Intent(this, NoteEditActivity.class);
            shortcutIntent.setAction(Intent.ACTION_VIEW);
            //创建一个Intent对象shortcutIntent，指定其目标为NoteEditActivity类，并设置动作为Intent.ACTION_VIEW
            shortcutIntent.putExtra(Intent.EXTRA_UID, mWorkingNote.getNoteId());
            //将正在编辑的笔记ID作为附加数据放入shortcutIntent中。
            sender.putExtra(Intent.EXTRA_SHORTCUT_INTENT, shortcutIntent);
            sender.putExtra(Intent.EXTRA_SHORTCUT_NAME,
                    makeShortcutIconTitle(mWorkingNote.getContent()));
            //将笔记内容生成适合作为快捷方式图标标题的字符串，并放入sender中作为附加数据。
            sender.putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE,
                    Intent.ShortcutIconResource.fromContext(this, R.drawable.icon_app));
            sender.putExtra("duplicate", true);
            //将应用程序的图标资源作为快捷方式图标放入sender中。
            sender.setAction("com.android.launcher.action.INSTALL_SHORTCUT");
            //设置sender的动作为com.android.launcher.action.INSTALL_SHORTCUT，表示要安装快捷方式。
            showToast(R.string.info_note_enter_desktop);
            //弹出一个简短的提示消息，提示用户笔记已经进入桌面。
            sendBroadcast(sender);
            //发送广播，安装快捷方式。
        } else {
            /**
             * There is the condition that user has input nothing (the note is
             * not worthy saving), we have no note id, remind the user that he
             * should input something
             */
            Log.e(TAG, "Send to desktop error");
            showToast(R.string.error_note_empty_for_send_to_desktop);
            //如果当前正在编辑的笔记没有有效的笔记ID，则执行以下操作：
            //输出一个错误日志，表示发送到桌面出错
            //弹出一个提示消息，提醒用户必须输入一些内容才能发送到桌面。
        }
    }

    //用于生成快捷图标的标题
    private String makeShortcutIconTitle(String content) {
        content = content.replace(TAG_CHECKED, "");
        //使用replace()方法将文本中的所有TAG_CHECKED和TAG_UNCHECKED替换为空字符串，即去除已选中和未选中标记。
        content = content.replace(TAG_UNCHECKED, "");
        return content.length() > SHORTCUT_ICON_TITLE_MAX_LEN ? content.substring(0,
                SHORTCUT_ICON_TITLE_MAX_LEN) : content;
    }

    //用于显示一个短时间的 Toast 提示消息
    private void showToast(int resId) {
        showToast(resId, Toast.LENGTH_SHORT);
    }

    //用于显示一个指定时长的 Toast 提示消息
    private void showToast(int resId, int duration) {
        Toast.makeText(this, resId, duration).show();
    }
}

